package Stanford::Orange::Sendmail ;

## no critic (CodeLayout::ProhibitParensWithBuiltins) ;

use warnings ;
use strict ;
use autodie ;

use Carp ;
use Email::Sender::Transport::SMTP ;
use Email::Sender::Simple qw(sendmail) ;
use Authen::SASL qw(Perl) ;

sub new {
    my ($proto, %args) = @_ ;
    my $self = {} ;
    my $class = ref($proto) || $proto ;

    $self->{'smtp_host'}    = $args{'smtp_host'}    || 'smtp.stanford.edu' ;
    $self->{'smtp_port'}    = $args{'smtp_port'}    || '465' ;
    $self->{'authenticate'} = $args{'authenticate'} || 0 ;
    $self->{'username'}     = $args{'username'}     || undef ;
    $self->{'password'}     = $args{'password'}     || undef ;
    $self->{'transport'}    = $args{'transport'}    || undef ;
    $self->{'ssl'}          = $args{'ssl'}          || 1 ;
    $self->{'debug'}        = $args{'debug'}        || 0 ;
    $self->{'to'}           = $args{'to'}           || undef ;
    $self->{'cc'}           = $args{'cc'}           || undef ;
    $self->{'bcc'}          = $args{'bcc'}          || undef ;
    $self->{'from'}         = $args{'from'}         || undef ;
    $self->{'subject'}      = $args{'subject'}      || undef ;
    $self->{'body'}         = $args{'body'}         || undef ;

    bless($self, $class) ;

    $self->initialize() ;

    return $self ;
}

sub initialize {
    my $self = shift ;

    # Set up the transport
    my $transport ;
    if ($self->{'authenticate'}) {
        $transport = Email::Sender::Transport::SMTP->new(
            {
                host          => $self->{'smtp_host'},
                port          => $self->{'smtp_port'},
                sasl_username => $self->{'username'},
                sasl_password => $self->{'password'},
                ssl           => $self->{'ssl'},
                debug         => $self->{'debug'},
            }
        ) ;
    } else {
        $transport = Email::Sender::Transport::SMTP->new(
            {
                host  => $self->{'smtp_host'},
                port  => $self->{'smtp_port'},
                ssl   => $self->{'ssl'},
                debug => $self->{'debug'},
            }
        ) ;
    }
    $self->{'transport'} = $transport ;

    return ;
}

sub make_email {
    my $self = shift ;

    my $to      = $self->{'to'} ;
    my $from    = $self->{'from'} ;
    my $subject = $self->{'subject'} ;
    my $body    = $self->{'body'} ;

    if (!$to) {
        croak q[missing 'To:' address] ;
    }
    if (!$from) {
        croak q[missing 'From:' address] ;
    }
    if (!$subject) {
        croak 'missing Subject' ;
    }

    my @headers = (
        From    => $from,
        To      => $to,
        Subject => $subject,
    ) ;

    if ($self->{'cc'}) {
        my $cc_formatted = join(q{,}, @{ $self->{'cc'} }) ;
        push(@headers, ('Cc', $cc_formatted)) ;
    }

    my $email = Email::Simple->create(
        header => \@headers,
        body   => $self->{'body'},
    ) ;

    return $email ;
}

sub deliver {
    my $self = shift ;

    my $email = $self->make_email() ;

    my $transport = $self->{'transport'} ;
    return sendmail($email, { transport => $transport }) ;
}

1 ;
